//mpakdump
//by thakis
//v1.0
//v2.0 - 2009/10/06 - modified to add support for Metroid Prime 3 - by revel8n
//v2.1 - 2009/11/01 - Final modified by Antidote
#include "main.h"

using namespace std;
int mpakMake(const char* dir,int version);
bool bQuiet = false;
bool bVerbose = false;

void dumpFile(const std::string& name, FILE* src, u32 offset, u32 length)
{
    int t = ftell(src);
    fseek(src, offset, SEEK_SET);

    FILE* out = fopen(name.c_str(), "wb");
    if (out == NULL)
        return;

    u32 read = 0;
    u8 buff[1024];
    while (read < length)
    {
        int r = fread(buff, 1, min(1024u, length - read), src);
        fwrite(buff, 1, r, out);
        read += r;
    }

    fclose(out);
    fseek(src, t, SEEK_SET);
}

Statistics mpakDump(FILE* f, const std::string& name)
{
    MPakHeader h;
    Statistics out;
    fseek(f, 0, SEEK_SET);

    fread(&h, sizeof(h), 1, f);
    toWORD(h.version1);
    toWORD(h.version2);
    toDWORD(h.size);

    if (h.version1 == 3 && h.version2 == 5)
    {
        out = mpakDump_v1(f, name);
    }
    else if (h.version1 == 0 && h.version2 == 2)
    {
        out = mpakDump_v3(f, name);
    }
    else
    {
        cout << "Unknown pak file version!" << endl;
        out.numFiles = 0;
    }
    return out;
}

void message()
{
    cout << string(70, '=') << endl << endl;
    cout << "\tWelcome to MPakDump the Metroid Prime PakDumper" << endl;
    cout << string(70, '=') << endl << endl;
    cout << "\tWritten by Thakis, Revel8n, interdpth, and Antidote" << endl << endl;
}
void usage()
{
    cout << "Usage: mpakdump -f <file.pak> [options]" << endl;
    cout << "Current Options:" << endl;
    cout << "-f or -info\t-- Show basic information about the file (does not dump)" << endl;
    cout << "-f or -file\t-- Specifies the file to dump" << endl;
    cout << "-v or -verbose\t-- Displays the processs in more detail" << endl;
    cout << "-q or -quiet\t-- Don't print anything" << endl;
    //cout << "-m or -make\t-- build a pak file syntax -m version dirName" << endl;
    cout << endl;
}

void fileinfo(string filename)
{
    FILE* f = fopen(filename.c_str(), "rb");

    MPakHeader h;

    fseek(f, 0, SEEK_SET);

    fread(&h, sizeof(h), 1, f);
    toWORD(h.version1);
    toWORD(h.version2);
    toDWORD(h.size);

    cout << "PakVersion: ";

    if (h.version1 == 3 && h.version2 == 5)
        cout << "Metroid Prime 1 or 2" << endl;
    else if (h.version1 == 0 && h.version2 == 2)
        cout << "Metroid Prime 3" << endl;
    else
    {
        cout << "unknown version or invalid file: " << filename << endl;
        return;
    }
}
int exit()
{
    cout << "\t\t Press Enter to Exit" << endl;
    cin.get();
    return 0;
}
int returnfilesize(FILE* fp)
{
    int add=ftell(fp);
    fseek(fp,0,SEEK_END);
    int ret = ftell(fp);
    fseek(fp,add,SEEK_SET);
    return ret;

}
int main(int argc, char* argv[])
{
    FILE* f;
    bool bHaveFile = false;
    bool bShowInfo = false;
    //bool bMakePak = false;
    std::string dirName;
    //int version;
    string fname;
    Statistics out;
    if (argc < 2)
    {
        message();
        usage();
        return exit();
    }
    // Very simple command line interface (need to change this)
    for (int i = 1; i < argc; i++)
    {
        if (!stricmp(argv[i], "-info") || !stricmp(argv[i], "-i"))
        {
            bShowInfo = true;
            if (bHaveFile)
            {
                fileinfo(fname);
            }
        }
        if ((!stricmp(argv[i], "-file") || !stricmp(argv[i], "-f")) && !bHaveFile)
        {
            if (argv[i+1] != 0)
            {
                fname = argv[i+1];
                bHaveFile = true;

                if (bShowInfo)
                {
                    fileinfo(fname);
                }
            }
        }
        if (!stricmp(argv[i], "-quiet") || !stricmp(argv[i], "-q"))
            bQuiet = true;
        if (!stricmp(argv[i], "-verbose") || !stricmp(argv[i], "-v"))
            bVerbose = true;
        /*if (!stricmp(argv[i], "-make") || !stricmp(argv[i], "-m"))
        {
            bMakePak = true;
            cout << "What Version: ";
            cin >> version;
            dirName = argv[i+1];
        }*/
    }

    if (bHaveFile && !bShowInfo)
    {
        dirName = fname;
        dirName = dirName.erase(dirName.find_last_of('.'), 4);

        mkdir(dirName.c_str());
        if ((f = fopen(fname.c_str(), "r+b")) == NULL)
        {
            cout << "File Not Found" << fname << endl;
            return exit();
        }
        out = mpakDump(f, dirName);

        fclose(f);

        if (!bQuiet && out.numFiles > 0)
        {
            cout << string(70, '=') << endl << endl;
            cout << "\t" << out.numNames << " Names " << out.numFiles << " Files " << out.numCompressed << " Compressed" << endl;
        }
    }
    /*else if (bMakePak)
    {
        mpakMake(dirName.c_str(), version);
    }*/
    else if (!bShowInfo)
    {
        message();
        cout << "Missing required option -f <file.pak>" << endl << endl;
        usage();
    }

    if (!bQuiet)
        return exit();

    return 0;
}
/*
int mpakMake(const char* dir,int version)
{
    char filename[256]={0};
    FILE* fp=NULL;
    int i=0;
    for (i=strlen(dir);i>0;i--)
    {
        if (dir[i]=='\\')
        {
            i++;
            break;

        }
    }
    if (i>0)
    {
        sprintf(filename,"%s.Pak",dir);
        fp=fopen(filename,"w+b");
        if (fp)
        {
            printf("File opening was a sucess!\n");
            if (version>2)
            {
                //		 MakePakMP3(dir,fp);	//MP3
            }
            else //Mp2 and MP1
            {
                printf("Creating Pak file.\n");
                makePakMP1(dir,fp);
            }
        }
        else
        {
            cout << "Failed to open \'" << filename << "\'" << endl;
        }
    }
    else
    {
        printf("Invalid Directory\n");
        return 0;
    }
    return 0;

}
*/
